/* *******************
 * browser.js
 * Firefox/Mozilla specific code for QuickNote   <http://quicknote.mozdev.org/>
 * Used in overlay.xul (F/M specific overlays)
 * Requires overlay.js (common overlay code)
 *
 * Author: Jed Brown (quicknote@jedbrown.net)
 * Contributor(s):
 *   Nickolay Ponomarev (asqueella@gmail.com)
 *   Someone from Download Manager Tweak <dmextension.mozdev.org> team
 ******************** */

// do the init on load
window.addEventListener("load", QuickNote_initOverlay, false);

function QuickNote_initOverlay(){
  var menu = document.getElementById("contentAreaContextMenu");
  menu.addEventListener("popupshowing",QuickNote_contextPopupShowing,false);
  QuickNote_updateKeys();
  QuickNote_autoStart();
}

function QuickNote_autoStart() {
  if(getQNPrefs().get("autostart"))
  { // check if need to open QN:
    if(QN_Enumerator.EnumQNWindows(function() {return false;}))
      QuickNote_openQuickNote(true);
  } // todo: if openintab is default, open a tab even if it not a new window.
    // dep. on bug about data sync.
}

// Opens QuickNote depending on set pref
function QuickNote_openQuickNote(aForceOpen, aText) {
  var openMethod = getQNPrefs().get("openqn");
  switch(openMethod) {
    case QN_SIDEBAR:
      toggleSidebar("viewQuickNoteSidebar", aForceOpen);
      break;
/* We don't support opening QN in tab, since XPCNW broke the "send to" functionality.
    case QN_TAB:
      QuickNote_openQuickNoteInTab(aForceOpen);
      break;
*/
    default:
      openQNFloat(aText);
  }
}

// Open QuickNote as a tab
//### taken from Download Manager Tweak dmextension.mozdev.org
function  QuickNote_openQuickNoteInTab(forceOpen){
  var numTabs = gBrowser.mTabContainer.childNodes.length;
  var index = 0;
  var oldTab = getBrowser().selectedTab;

  if (oldTab.getAttribute('QuickNote') == 'true') {
  if (getWebNavigation().currentURI.spec == "chrome://quicknote/content/quicknote.xhtml") {
      if(!forceOpen)
        BrowserCloseTabOrWindow();
      return;
    }
    else oldTab.removeAttribute('QuickNote');
  }

  var tempTab = null;
  while (index < numTabs) {
    tempTab = gBrowser.mTabContainer.childNodes[index];

    if (tempTab.getAttribute('QuickNote') == 'true') {
      if (tempTab.label == 'QuickNote' || tempTab.label == 'chrome://quicknote/content/quicknote.xhtml'){
        getBrowser().selectedTab = tempTab;
        break;
      }else {
        getBrowser().selectedTab = tempTab;
        var url = getWebNavigation().currentURI.spec;
        if (url == 'chrome://quicknote/content/quicknote.xhtml') break;
        else tempTab.removeAttribute('QuickNote');
    }
  }
    else index = index+1;
  }

  // If no QuickNote tab is open, then create one
  if (index == numTabs) {
    var QuickNoteTab = gBrowser.addTab('chrome://quicknote/content/quicknote.xhtml');
    QuickNoteTab.setAttribute('QuickNote', 'true');
    getBrowser().selectedTab = QuickNoteTab;
  }
}


// Called before closing sidebar. Used to show the save confirmation dialog.
// Throws an exception if closing was cancelled.
function QN_onBeforeToggleSidebar()
{
  if(document.getElementById('viewQuickNoteSidebar').hasAttribute('checked'))
    return; // the sidebar is being opened

  QuickNote_setupDocument(document.getElementById("sidebar").contentDocument);
  if(!QuickNote_onClose())
  { // I couldn't find an elegant way to prevent sidebar closing.
    // (it is can't be done here for sure - see browser.js/toggleSidebar() --
    // maybe there is other way - without using 'observes' - to handle
    // 'onBeforeClose'...)
    throw "QuickNote: Don't close sidebar!";
  }
}


function QuickNote_ActivateQNTab(doc) {
  var browser = doc.getElementById('content');
  var tempTab = browser.mTabContainer.firstChild;
  var index = 0;
  while (tempTab) {
    if (tempTab.getAttribute('QuickNote') == 'true') {
      getBrowser().selectedTab = tempTab;
      return true;
    }
    index ++;
    tempTab = tempTab.nextSibling;
  }
  return false;
}

// This gets the selected text (aText) from the contentArea and
// adds it to one of the QN tabs (respects the value of quicknote.totabint).
// If there is a QN window (or a tab, or a sidebar) open already, we're
// focusing it. Otherwise we open a new window.
function QuickNote_SendSelectedTextToQN(aText) {
  // QuickNote tab that catches the text:
  var tabtosendto = getQNPrefs().get("totabint");

  // First, check for sidebar in current window
  var qndoc = QN_Enumerator.FindQNSidebar(document);

  // Then check for an existing floating window
  if(!qndoc) {
    qndoc = QN_Enumerator.FindQNFloat();
    if(qndoc) {
      var qnWindow = qndoc.defaultView;
      qnWindow.focus();
    }
  }

  // Finally try to find a QN tab in current window
  if(!qndoc) {
    qndoc = QN_Enumerator.FindQNTab(document);
    if(qndoc) {
      QuickNote_ActivateQNTab(document)
    }
  }

  if(qndoc) {
    QuickNote_setupDocument(qndoc);
    QuickNote_catchText(aText);  // todo: handle position reset!!
  } else {
    QuickNote_sendText(aText);
    QuickNote_openQuickNote(true);
  }
}

// determine which context menu items are shown
function QuickNote_contextPopupShowing() {
  var isTextSelectionQN = false;

  if(gContextMenu)
    isTextSelectionQN = gContextMenu.isTextSelected;

  var qnmenuitem = document.getElementById("quicknote-contexttab");
  if(qnmenuitem)
    qnmenuitem.hidden = !isTextSelectionQN;

  var isTextSelectionLinkQN = false;

  if(gContextMenu)
    isTextSelectionLinkQN = gContextMenu.onLink;

  var qnmenuitemlink = document.getElementById("quicknote-contexttablink");
  if(qnmenuitemlink)
    qnmenuitemlink.hidden = !isTextSelectionLinkQN;
}

