/* *******************
 * preferences.js
 * Default preferences for QuickNote  <http://quicknote.mozdev.org/>
 *
 * Author: Jed Brown <quicknote@jedbrown.net>
 * Contributor(s):
 *   Nickolay Ponomarev <asqueella@gmail.com>
 *
 ******************** */

const QN_notesCount = 4;

const QN_SIDEBAR = 0;
const QN_FLOAT = 1;
const QN_TAB = 2;


function getQNPrefs() {
  return QuickNotePrefs;
}

var QuickNotePrefs = {
  gPreferences: null,
  getPreferences: function() {
    if(!this.gPreferences) {
      try {
        this.gPreferences = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefService).
                            getBranch("quicknote.");
      } catch (e) {}
    }
    return this.gPreferences;
  },

  types: {
    /*bool*/  firststart: 0, hidebuttonbox: 0, showUrl: 0,
    /*int*/   lasttabopen: 1, numtabs: 1, openqn: 1, totabint: 1,
              autosave: 1, fontsize: 1, // int
    /*char*/  charset: 2, fontname: 2, fontcolor: 2, userstyle: 2, bgcolortab: 2,
    /*unichar*/ tabname: 3, tabpath: 3, senttext: 3
    },

  defaults: { // xxx we should get rid of that and use
              // nsIPrefService.getDefaultBranch()'s data (coming from
              // defaults/preferences)
    firststart: true,
    hidebuttonbox: true,
    showUrl: true,
    lasttabopen: 1,
    numtabs: 2,
    openqn: QN_FLOAT,
    totabint: 1,
    charset: "UTF-8",
    //autosave options:
    // Value   Desc
    //  >0      autosave every X chars & on close
    //  =0      do not autosave while typing
    //  <0      no autosave at all
    autosave: 10,
    
    fontsize: 12,
    fontname: "Verdana,sans-serif",
    fontcolor: "Black",
    userstyle: "background-image:none !important;",
    bgcolortab: "#FFFF99",
    tabname: "Note ",
    tabpath: "",
    senttext: undefined
    },
    
  // for note-specific prefs; example: ("fontcolor", 1) -> "fontcolor1"
  getRealPrefName: function(aPrefName, aIndex) {
    if(aIndex) {
      if(aPrefName == "tabname")
        return "tab" + aIndex + "name";
      if(aPrefName == "tabpath")
        return "tab" + aIndex + "path";
      return aPrefName + aIndex;
    }
    return aPrefName;
  },

  // get pref. Example of usage:
  //    var openqn = QuickNotePrefs.get("openqn");
  //    var fontcolor1 = QuickNotePrefs.get("fontcolor", 1);
  get: function(aPrefName, aIndex) {
    var realPrefName = this.getRealPrefName(aPrefName, aIndex);
    try {
      var pref = this.getPreferences();
      switch(this.types[aPrefName]) {
        case 0:
          return pref.getBoolPref(realPrefName);
        case 1:
          return pref.getIntPref(realPrefName);
        case 2:
          return pref.getCharPref(realPrefName);
        case 3:
          return pref.getComplexValue(realPrefName,
            Components.interfaces.nsIPrefLocalizedString).data;
      }
    } catch(e) {}
    if(aPrefName == "tabname") {
      var tabname;
      try {
        tabname = QN_stringBundle.getFormattedString("default.notetitle", [aIndex]);
      } catch(e) {
        tabname = this.defaults.tabname + "&"+aIndex;
      }
      return tabname;
    }
    return this.defaults[aPrefName];
  },

  // set pref
  set: function(aPrefName, aValue, aIndex) {
//    try {  //shouldn't really fail here, so we avoid using try..catch here
      var realPrefName = this.getRealPrefName(aPrefName, aIndex);
      var pref = this.getPreferences();
      dump(realPrefName + "->" + aValue);
      switch(this.types[aPrefName]) {
        case 0:
          pref.setBoolPref(realPrefName, aValue);
          break;
        case 1:
          pref.setIntPref(realPrefName, aValue);
          break;
        case 2:
          pref.setCharPref(realPrefName, aValue);
          break;
        case 3:
          var str = Components.classes["@mozilla.org/supports-string;1"]
                      .createInstance(Components.interfaces.nsISupportsString);
          str.data = aValue;
          pref.setComplexValue(realPrefName,
                                 Components.interfaces.nsISupportsString, str);
          break;
      }
//    } catch(e) {}
  },

  // remove given userpref
  clear: function(aPrefName, aIndex) {
    var realPrefName = this.getRealPrefName(aPrefName, aIndex);
    try {
      this.getPreferences().clearUserPref(realPrefName);
    } catch(e) {
    }
    
  }
};

