 /* ***************
// quicknote.js
// Main implementation file for QuickNote  <http://quicknote.mozdev.org/>
//
// Author: Jed Brown (quicknote@jedbrown.net)
// Additional code taken from Cdn's Chrome Edit extension.
// Contributors:
//   Nickolay Ponomarev (asqueella@gmail.com)
//   Leung WC
//   ...
****************** */

var QN_stringBundle;
// quicknote's document object (inserted here, because simple
// 'document.something' doesn't work when a function is
// called from a global overlay (such as overlay.xul) )
var QN_Document=null;


var QN_charsLeft = Array(QN_notesCount+1); // ..until autosave (when qnautosave>0)

// Desc: QN_noteLength[X] = PadX.textLength
//       Used in autosave impl.
// Note: Do not rely on this prop. value (since it isn't always in sync with
// PadX.textLength).
var QN_noteLength = Array(QN_notesCount+1);


////////////////////////////////////////////////////////////////////////////////
// OnLoad
//
// Desc: Starts everyting (loading, Tab Labels, caught text, etc.)
// Parameters: Pad to copy the selection (via arguments[0])
// Returns: nothing
////////////////////////////////////////////////////////////////////////////////
function QuickNote_OnLoad() {
// TODO -p5: find out what is the default state for *.collapsed
// (asq) actually I thought it was true for 3rd & 4th; false for 1st and 2nd.
// I also suspect that each time OnLoad is triggered, the new instance of .xul/.js and therefore
// of all the objects, including textboxes is created, so some of these statements /are/ redundant
// I was too lazy, however, to check that out, so I ended up adding this comment ;)
// (jed) No worries, it's not really a priority, well figure it out sooner or later.


//-- set up XUL attributes depending on prefs
  QuickNote_setupDocument(document);
  //Set up Spellbound if installed for SpellChecking -> Thanks Robert!
  setupSpellBound();
/*  if(getQNPrefs().get("firststart")) {
    //to remove:
    // xxx padfontsize is not set correctly here at first time
    var alertText = QN_stringBundle.getString("qn.welcome");
    alert(alertText);
    window.close();
    window.open('qnsettings.xul', 'QuickNote - Settings',
                'chrome,centerscreen,resizable');
    return;
  }*/

  var i;
  if(!getQNPrefs().get("hidebuttonbox")) {
    for(i=1; i<=QN_notesCount; i++)
      window.document.getElementById("buttonbox"+i).hidden = false;
  }

  var numberOfTabs = getQNPrefs().get("numtabs");
  for(i=1; i<=numberOfTabs; i++)
    document.getElementById("Tab"+i).collapsed = false;
  for(i=numberOfTabs+1; i<=QN_notesCount; i++)
    document.getElementById("Tab"+i).collapsed = true;

//-- load notes
  for(i=1; i<=QN_notesCount; i++)
    QuickNote_loadNote(i);

  // now catch the text sent to QuickNote via context menu item
  // "Send .. to QuickNote" (if any [text])
  var textRecipent = null; // for now; see catchText comment
  try {
    if(("arguments" in window) && (0 in window.arguments) )
      textRecipent = QuickNote_catchText(window.arguments[0]);
    else
      textRecipent = QuickNote_catchText();
  } catch(e) {
    Components.utils.reportError(e)
  }

  if(!textRecipent) {
    // Gets the last tab that was opened and sets
    // the tab to focus via selectedIndex
    var lastTab = getQNPrefs().get("lasttabopen");
    document.getElementById("qntabs").selectedIndex = lastTab - 1;
    var thePadId = "Pad" + lastTab;
    var thePad = document.getElementById(thePadId);
    thePad.focus();
  } // otherwise focus goes to the tab that caught the text
}

// this function must be called from global overlays (ie. those that do not
// overlay the QuickNote window, eg. overlay.xul) prior to any other qn.js
// functions.
function QuickNote_setupDocument(aDoc)
{
  QN_Document = aDoc;
  QN_stringBundle = aDoc.getElementById("strings");
//  QN_synchronize(aDoc);
}

function QuickNote_closeWindow()
{
  window.close();
}

function QuickNote_getTextareaText(Note) {
  return QN_Document.getElementById('Pad'+Note).value;
}

function QuickNote_setTextareaText(Text, Note) {
  QN_Document.getElementById('Pad'+Note).value = Text;
}


// applies visual customizations for a tab (bgcolor, fonts, tab names etc.)
function QuickNote_applyCustomizations(Note) {
  var NodeID = "Pad" + Note;
  var thePad = QN_Document.getElementById(NodeID);
  var oldStyles = thePad.getAttribute("style");
  var pref = getQNPrefs();

  thePad.setAttribute("style", oldStyles + " " + "font: " +
    pref.get("fontsize",Note).toString() + "px " +
    pref.get("fontname",Note) + "; " +
    "color:" + pref.get("fontcolor",Note) + "; " +
    "-moz-appearance: none; background-color: " +
    pref.get("bgcolortab",Note) + "; " +
    pref.get("userstyle",Note));

  QN_renameTab(Note);
}

////////////////////////////////////////////////////////////////////////////////
// loadNote()
//
// Desc: This function decides whether to load user defined files or create
//       new quicknote[1-4].txt in the profile chrome directory.
// Params: Note - which note to load (1..4)
////////////////////////////////////////////////////////////////////////////////
function QuickNote_loadNote(Note) {
  var userTabpath = getQNPrefs().get("tabpath", Note);
  if(!userTabpath)
    QuickNote_setDefaultFile(Note);
  else
    QuickNote_loadAFile(userTabpath, Note);
  QuickNote_applyCustomizations(Note);
}

////////////////////////////////////////////////////////////////////////////////
// setDefaultFile()
//
// Desc: Used for loading the default Profile Notes, sets the dir and calls
//       loadFromFile() to actually load the files.
// Parameters:
//    Note: Where it is headed to (1..4)
////////////////////////////////////////////////////////////////////////////////
function QuickNote_setDefaultFile(Note) {//!! review/cleanup
  var targetNodeID = "Pad" + Note;
  var fileName = "quicknote" + Note + ".txt";
  var f = DirIO.get('UChrm');
  f.append(fileName);

  if(f.exists()) // todo: f may be a directory
  {  // if file already exists then just open it.
    QuickNote_loadFromFile(f,Note);
    getQNPrefs().set("tabpath", f.path, Note);
  } else if(!f.exists())
  { // If it doesn't exist, create the file, then make sure it exists, then open.
    FileIO.create(f);
    if (f.exists()) {
      QuickNote_loadFromFile(f,Note);
      getQNPrefs().set("tabpath", f.path, Note);
    } else {
      var textAlert =
        QN_stringBundle.getFormattedString("qnerr.cantCreate", [f.path])+'\n'+
        QN_stringBundle.getString("qnerr.cantCreate.hint");
      QuickNote_setTextareaText(textAlert, targetNodeID);
    }
  }
}


////////////////////////////////////////////////////////////////////////////////
// loadAFile
//
// Desc: Loads userdefined files (set in prefs)
// Parameters:
//    userFileName: The name and location of the file to load.
//    Note: Where it is headed to (1..4)
// Returns: true or false
////////////////////////////////////////////////////////////////////////////////
function QuickNote_loadAFile(userFileName, Note) {
  var textAlert;
  if(!userFileName) {
    //!! these statements never get executed
    textAlert = QN_stringBundle.getString("qnerr.noFile");
    QuickNote_setTextareaText(textAlert, Note);
    return false;
  }

  var f = FileIO.open(userFileName);

  if (f.exists()) {  //If file already exists then just open it.
    QuickNote_loadFromFile(f, Note);
  } else { //If it doesn't exist, tell the user
           // todo: maybe try to create the file instead?
    textAlert =
      QN_stringBundle.getFormattedString("qnerr.invalidFile", [f.path])+'\n'+
      QN_stringBundle.getString("qnerr.invalidFile.hint");
    QuickNote_setTextareaText(textAlert, Note);
  }
  return true;
}

////////////////////////////////////////////////////////////////////////////////
// loadFromFile
// Desc: Actually loads the file thanks to many people, including JSLib team,
//       MonkeeSage, Leung WC, asqueella, ...
// Parameters:
//    f: nsIFile
//    Note: where the file should be loaded to (1..4)
// Returns: true if file saved, false otherwise
////////////////////////////////////////////////////////////////////////////////
function QuickNote_loadFromFile(f,Note) {
  var pref = getQNPrefs();
  var charset = pref.get("charset");
  QN_charsLeft[Note] = pref.get("autosave");
  try {
    var stuff = FileIO.read(f, charset);
    QuickNote_setTextareaText(stuff,Note);
    QN_noteLength[Note] = QN_Document.getElementById("Pad"+Note).textLength;
  } catch(e) {
    var textAlert =
      QN_stringBundle.getFormattedString("qnerr.failedLoadFile", [f.path])+'\n'+
      QN_stringBundle.getFormattedString("qnerr.failedLoadFile.hint", [charset]);
    QuickNote_setTextareaText(textAlert,Note);
  }
}



////////////////////////////////////////////////////////////////////////////////
// renameCurrentTab
//
// Desc: Rename the current Tab
// Sets prefs quicknote.tab1name, .tab2name, etc.
////////////////////////////////////////////////////////////////////////////////
function QuickNote_renameCurrentTab() {
  var Note = QN_Document.getElementById("qntabs").selectedIndex+1;//current note number
  var pref = getQNPrefs();
  
  var currentTabName = pref.get("tabname", Note);
  var displayName = QN_parseTabName(currentTabName)[0];
  var promptText = QN_stringBundle.getFormattedString("qn.tabRenamePrompt", [displayName]);
  var newTabName = prompt(promptText, currentTabName);
  if(newTabName != null) { // Set up new name
    pref.set("tabname", newTabName, Note);
    QN_renameTab(Note);
  }
}

// '&' specifies accesskey
// '&&' -> '&' in the title; '&&&' -> '&', accesskey.
// returns: [display name, acesskey]
function QN_parseTabName(aName) {
//  var a = /^(.*[^&]|[^&]*)&([^&]|&&)(.*)$/.exec(aName); //go figure what this means :P
  var a = RegExp("^(([^&]|&&[^&])*)" + "&([^&]|&&)" + "(.*)").exec(aName);
  var accesskey, label;
                //      2       1          3           4
  if(a) {
    if(a[3] == '&&') a[3] = '&';
    if(!a[1]) a[1] = "";
    if(!a[4]) a[4] = "";

    accesskey = a[3];
    label = a[1] + a[3] + a[4];
  } else {
    label = aName;
    accesskey = "";
  }
  label = label.replace(/&&/g, "&");
  return [label, accesskey];
}


// rename tab #Note (the new title is read from user prefs)
function QN_renameTab(Note) {
  var Name = getQNPrefs().get("tabname", Note);
  var tab = QN_Document.getElementById("Tab"+Note);

  var a = QN_parseTabName(Name);
  if(!a[1]) a[1]=Note;
  
  // handle duplicate access keys
  // Will have to change it if we allow infinite number of notes.
  for(var i=1; i<=QN_notesCount; i++) {
    var t = QN_Document.getElementById("Tab"+i)
    if(t.getAttribute("accesskey") == a[1]) {
      if(i < Note)
        tab.removeAttribute("accesskey");
      else if(i > Note)
        t.removeAttribute("accesskey");
    }
  }

  tab.setAttribute("accesskey",a[1]);
  tab.label = a[0];
}

// returns the index (1..4) of current note
function QN_currentNote()
{
  // todo: use this whenever possible
  return QN_Document.getElementById("qntabs").selectedIndex+1;//selIndex is 0-based!
}

////////////////////////////////////////////////////////////////////////////////
// saveCurrent
//
// Desc: Saves the active pad to a corresponding file through a call to saveNote
//       Used for Ctrl+S ('Save Current Note') handling.
////////////////////////////////////////////////////////////////////////////////
function QuickNote_saveCurrent()
{
  QuickNote_saveNote(QN_Document.getElementById("qntabs").selectedIndex+1);
}


////////////////////////////////////////////////////////////////////////////////
// saveAll
// Desc: Saves all tabs using SaveNote
////////////////////////////////////////////////////////////////////////////////
function QuickNote_saveAll()
{
  for(var i=1; i<=QN_notesCount; i++)
    QuickNote_saveNote(i);
}

////////////////////////////////////////////////////////////////////////////////
// saveNote
//
// Desc: Saves a note. Utilized the saveAFile function for actual saving
// Parameters: Note - the number of the note to be save (1..4)
// Returns: nothing
////////////////////////////////////////////////////////////////////////////////
function QuickNote_saveNote(Note) {
  var fName = getQNPrefs().get("tabpath", Note);
  var p = QuickNote_saveAFile(fName, Note);
  if(p) {
    QN_charsLeft[Note] = getQNPrefs().get("autosave");
    QN_Document.getElementById('save'+Note).setAttribute('disabled', 'true');
  }
}

////////////////////////////////////////////////////////////////////////////////
// autoSave()
//
// Desc: Auto-save on close function.
//       Saves all the [modified] files via saveNote().
//       Also saves the active tab information.
////////////////////////////////////////////////////////////////////////////////
function QuickNote_autoSave() {
  getQNPrefs().set("lasttabopen",
    QN_Document.getElementById("qntabs").selectedIndex+1);//note that selIndex is 0-based.

  if(getQNPrefs().get("autosave") < 0)
    return;

  for(var i=1; i<=QN_notesCount; i++) {
    if (!QN_Document.getElementById('save'+i).hasAttribute('disabled'))
      QuickNote_saveNote(i);
  }
}

////////////////////////////////////////////////////////////////////////////////
// onClose()
//  used when 'no autosave' option is selected
//  shows a dialog to let user select the notes he wishes to save
////////////////////////////////////////////////////////////////////////////////
function QuickNote_onClose()
{
  // since this file (and this function in particular) is used in global
  // overlay (overlay.xul), we need to check if the prefs have changed.
  var pref = getQNPrefs();
  var autosave = pref.get("autosave");

  if(autosave < 0) {
    // we get ok/cancel return value from the dialog through re.val
    var re = {};
    var Show = Array(5);  //whether the i-th note was available in QuickNote
    var Save = Array(5);  //was it modified?
    var Changed = false;  //anything modified?
    var i;

    var numberOfTabs = pref.get("numtabs");
    for(i=1; i<=numberOfTabs; i++)    Show[i] = true;
    for(i=numberOfTabs+1; i<=4; i++)  Show[i] = false;

    var tabNames = [""];
    for(i=1; i<=4; i++)
    {
      tabNames.push(QN_parseTabName(pref.get("tabname", i))[0]);
      Save[i] = !QN_Document.getElementById('save'+i).hasAttribute('disabled');
      Changed = Changed || Save[i];
    }
    if(!Changed) return true; // everything is already saved => close right away

    window.openDialog("chrome://quicknote/content/qnconfirmsave.xul",
      "qnconfirmsave","chrome,modal,centerscreen",re,Show,tabNames,Save);
    if(!re.val)       // cancel: don't close QuickNote
      return false;
    else {            // ok: save the selected notes
      for(i=1; i<=4; i++)
        if(Save[i])
          QuickNote_saveNote(i);
    }
  }
  return true;
}

// onInput()
// Desc: "autosave every X characters" function.
// Note: deltaLen et al are used to handle pasting large text blocks
// from clipboard correctly and reliably :)
function QN_onInput(Note) {
  var thePad = QN_Document.getElementById("Pad" + Note);
  if(getQNPrefs().get("autosave") > 0)
  { // if autosave "as you type" is enabled...
    var deltaLen = Math.abs(QN_noteLength[Note]-thePad.textLength);
    QN_charsLeft[Note] = QN_charsLeft[Note] - Math.max(deltaLen,1);
    if(QN_charsLeft[Note] <= 0) // it's time to auto-save the note
      QuickNote_saveNote(Note);
    else
      QN_Document.getElementById('save' + Note).removeAttribute('disabled');
  } else
    QN_Document.getElementById('save' + Note).removeAttribute('disabled');
  QN_noteLength[Note] = thePad.textLength;
}


////////////////////////////////////////////////////////////////////////////////
// saveNoteAs
// Desc: Saves a Note to a user-defined location via saveAFile().
// Utilized saveAFile for actual saving.
//
// Parameters: Note: What to save (number, 1..4)
// Returns: true if list saved, false otherwise
//
// Originally Modeled after the SaveAs code from Tagzilla.mozdev.org!
////////////////////////////////////////////////////////////////////////////////
function QuickNote_saveNoteAs(Note) {
  //var f = DirIO.get('UChrm');
  //var myUserDirPlease = f.path;
  // todo: Note -> note title
  var promptText = QN_stringBundle.getFormattedString("qn.saveAsPrompt", [Note]);
  var fName = QuickNote_txtFilePicker(promptText, 1);
  if(fName == null)
    return false;

  var isSaved = QuickNote_saveAFile(fName, Note);
  return isSaved;
}


////////////////////////////////////////////////////////////////////////////////
// saveAFile
// Desc: Actually does the filesaving thanks to JSLib
// Parameters:
//    aUrl: string name of file to save, in platform-specific or file:// format
//    Note: where the data is to save (1..4)
// Returns: true if file saved, false otherwise
////////////////////////////////////////////////////////////////////////////////
function QuickNote_saveAFile(aUrl, Note) {
  var aFile = aUrl;
  if(aUrl.substring(0,7) == "file://") {
    var fUtils = new FileUtils();
    aFile = fUtils.urlToPath(aUrl);
  }

  var f = FileIO.open(aFile);
  if(!f) {
    var alertText = QN_stringBundle.getFormattedString("qnerr.cantOpenw", [aFile]);
    alert(alertText);
    // todo: perhaps throw an exception to prevent closing QN and losing data that couldn't be saved?
    return false;
  }

  var p;
  var data = QuickNote_getTextareaText(Note);
  if(data)
    p = FileIO.write(f,data,'w',getQNPrefs().get("charset"));
  else
    p = FileIO.unlink(f) && FileIO.create(f);
  return p;
}

////////////////////////////////////////////////////////////////////////////////
// catchText
//
// Desc: Catch text sent via context menu. Append it to the note specified with
//       tabtosendto.
// Parameters: QuickNote window; text to catch
// Returns: the pab-id which got the text; null when no text to catch.
// ?? make it return true/false ?? the return value is only tested against null...
///////////////////////////////////////////////////////////////////////////////
function QuickNote_catchText(aText) {
  var pref = getQNPrefs();
  var target = pref.get("totabint"); // what node to send text to, 1..4
  var result = null;
  
  if(!aText) {
    try {
      aText = pref.get("senttext");
    } catch(e) {}
    pref.clear("senttext");
  }

  if(aText) {
    // This is causing the extra /n to apear; todo
    pref.set("lasttabopen", target);
    var Pad = QN_Document.getElementById("Pad" + target);
    Pad.value = Pad.value + "\n" + aText + "\n";
    if(pref.get("autosave") >= 0) 
      QuickNote_saveNote(target); 
    // current behaviour is to pop the quicknote window on 'send text'
    // so we will also switch to the tab, that 'caught' the text:
    QN_Document.getElementById("qntabs").selectedIndex = target-1;
    Pad.focus();

    result = "Pad" + target;
  }
  return null;
}



////////////////////////////////////////////////////////////////////////////////
// txtFilePicker
//
// Parameters:
//  aTitle: title to go on file picker window
//  aSave: 1 if picking file to save/overwrite, 0 if picking file to load
// No longer used. --> aStart: directory to start from No longer used.
// (kept as a resource)    This must be an nsILocalFile.  new Dir("foo") in jslib/io/dir.js will
//          do the trick, but you have to do it yourself.
// Returns:
//  Name of file picked, in URL format, or null if cancelled
// Original taken and modified from Tagzilla.mozdev.org!
////////////////////////////////////////////////////////////////////////////////
function QuickNote_txtFilePicker(aTitle, aSave) {
  var retVal = null;
  try {
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
    fp.init(window, aTitle, (aSave ? nsIFilePicker.modeSave : nsIFilePicker.modeOpen));
    fp.appendFilters(nsIFilePicker.filterAll | nsIFilePicker.filterText);

    var result=fp.show();
    if (result == nsIFilePicker.returnOK || result == nsIFilePicker.returnReplace)
      retVal=fp.file.path;
  }
  catch (e) {
  }
  return retVal;
}


// insert some text into current note at cursor position
function QN_insertText(aText) {
  var command = "cmd_insertText";
  try {
    var controller = QN_Document.commandDispatcher.getControllerForCommand(command);
    if (controller && controller.isCommandEnabled(command)) {
      controller = controller.QueryInterface(Components.interfaces.nsICommandController);
      var params = Components.classes["@mozilla.org/embedcomp/command-params;1"];
      params = params.createInstance(Components.interfaces.nsICommandParams);
      params.setStringValue("state_data", aText);
      controller.doCommandWithParams(command, params);
    }
  }
  catch (e) {
    dump("An error occurred executing the cmd_insertText command\n");
    dump(e+"\n")
  }
}

// insert current date/time into current note using QN_insertText()
function QN_insertDateTime() {
  var dateTimeStr = new Date().toLocaleString();
  QN_insertText(dateTimeStr);
}

// here we process the TAB key (insert '\t' char ven VK_TAB is pressed)
function QN_onKeyPress(event) {
  var modifiers = (event.ctrlKey?1:0) | (event.altKey?2:0) | (event.shiftKey?4:0);

  if((event.keyCode == event.DOM_VK_TAB) && (modifiers == 0))
    QN_insertText("\t");
  else
    return true; // we didn't handle the event
  return false;
}


// CTRL+F7 toggle
// xxx customizable hotkey
function QN_onGlobalKeyPress(event) {
  var modifiers = (event.ctrlKey?1:0) | (event.altKey?2:0) | (event.shiftKey?4:0);
  if((event.keyCode == event.DOM_VK_F7) && (modifiers == 1))
    return QN_hideSelf();
  else
    return true;
}
function QN_hideSelf() {
  // find out if QN is opened in tab, sidebar or a floating window
  var parent = QN_Document.defaultView.parent;
  var loc = parent.location;
  if(loc == "chrome://quicknote/content/quicknote.xul") // floating window
    window.blur();
  else 
    return true;
  return false;
    
/*  else if(loc == "chrome://quicknote/content/quicknote.xhtml") // tab
    alert(parent.parent.location)
    //parent.parent.QuickNote_openQuickNoteInTab();  
  else // probably, sidebar | XXX Mozilla!
    parent.toggleSidebar("viewQuickNoteSidebar");
*/
}


//Testing Print functions -Jed
function QuickNote_Print() {
  //floating window
  var re = [];
  window.openDialog("chrome://quicknote/content/qnprint.xul", "QN Print","chrome,modal,centerscreen",re,QuickNote_getTextareaText(1), QuickNote_getTextareaText(2), QuickNote_getTextareaText(3), QuickNote_getTextareaText(4));
}


var QN_Enumerator = {
  ////////////////////////////////////////////////////////////////////////////////
  // EnumQNWindows(aCallbackFunction)
  // Desc: looks for open QuickNote windows - one floating window, all the
  //       instances in sidebar and in tab. For each found window calls
  //              aCallbackFunction(document, type);
  //
  // (document is the XULDocument object, with document.location ==
  //           'chrome://quicknote/content/quicknote.xul';
  //  type     is 0 if the enumerated instance is in sidebar,
  //              1 if it is in a floating window,
  //              2 if it is in a tab)
  //       Further enumeration can be cancelled by returning false from the
  //       callback function.
  //
  // Returns: false,  if the enumeration was cancelled from callback function,
  //          true,   otherwise.
  EnumQNWindows : function(aCallbackFunction)
  {
    var qndoc = this.FindQNFloat();
    if(qndoc)
      if(!aCallbackFunction(qndoc, QN_FLOAT))
        return false;

    var windowmanager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
    windowmanager = windowmanager.QueryInterface(Components.interfaces.nsIWindowMediator);

    // Cycle through browser windows looking for QN sidebars & tabs
    var enumerator = windowmanager.getEnumerator('navigator:browser');
    while(enumerator.hasMoreElements())
    {
      var win = enumerator.getNext();
      win = win.QueryInterface(Components.interfaces.nsIDOMWindow);
      var doc = win.document;

      qndoc = this.FindQNSidebar(doc);
      if(qndoc)
        if(!aCallbackFunction(qndoc, QN_SIDEBAR))
          return false;

      qndoc = this.FindQNTab(doc);
      if(qndoc)
        if(!aCallbackFunction(qndoc, QN_TAB))
          return false;
    }
    return true;
  },

  // Three helper functions: FindQNFloat, FindQNSidebar and FindQNTab.
  // All return the XULDocument for the found QN instance, null if nothing found.
  // FindQNSidebar() and FindQNTab() get one param - the (browser chrome)
  // document to search in.
  FindQNFloat : function()
  {
    var windowmanager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
    windowmanager = windowmanager.QueryInterface(Components.interfaces.nsIWindowMediator);

    var win = windowmanager.getMostRecentWindow('quicknote:mainwindow');
    if(win) {
      win = win.QueryInterface(Components.interfaces.nsIDOMWindow);
      if(win)
        return win.document;
    }
    return null;
  },

  FindQNSidebar : function(aDoc)
  {
    var sidebarBox = aDoc.getElementById('sidebar-box');
    var sidebar,qndoc;
    // Firefox
    if(sidebarBox.getAttribute('sidebarcommand') == 'viewQuickNoteSidebar') {
      sidebar = aDoc.getElementById('sidebar');
      qndoc = sidebar.contentDocument;
      return qndoc;
    }

    // Seamonkey
    if(!this._sidebar_is_hidden(aDoc))
    {
      var sidebarPanels = aDoc.getElementById('sidebar-panels');
      var currentPanel = sidebarPanels.getAttribute('last-selected-panel');
      if(currentPanel.indexOf('chrome://quicknote/content/quicknote.xul') >-1)
      {
        sidebar = aDoc.getElementById(currentPanel);
        if(sidebar && (sidebar.localName != 'vbox'))
          sidebar = sidebar.nextSibling;
        if(sidebar) {
          var browser = sidebar.firstChild;
          while(browser) {
            if((browser.localName=='browser') && !browser.hasAttribute('hidden'))
              break;
            browser = browser.nextSibling;
          }
          if(browser) {
            qndoc = browser.contentDocument;
            return qndoc;
          }
        }
      }
    }
    return null;
  },

  FindQNTab : function(aDoc)
  {
    var browser = aDoc.getElementById('content');
    var tempTab = browser.mTabContainer.firstChild;
    var index = 0;
    while (tempTab) {
      if (tempTab.getAttribute('QuickNote') == 'true') {
        var parentDoc = browser.browsers[index].contentDocument; //qn.xhtml doc.
        var qnframe = parentDoc.getElementById('qnframe'); // frame with qn.xul
        if(qnframe) {
          var qndoc = qnframe.contentDocument;
          return qndoc;
        }
      }
      index ++;
      tempTab = tempTab.nextSibling;
    }
    return null;
  },

  // Helper function for QuickNote_EnumQNWindows [on Seamonkey]
  // copied from Mozilla's sidebarOverlay.js + added a check against Firefox
  _sidebar_is_hidden : function (aDoc) {
    var sidebar_title = aDoc.getElementById('sidebar-title-box');
    if(!sidebar_title) return true; // Firefox doesn't have 'sidebar-title-box'
                                    // Hope nobody else will use this id under Firefox.
    var sidebar_box = aDoc.getElementById('sidebar-box');
    return sidebar_box.getAttribute('hidden') == 'true' ||
           sidebar_title.getAttribute('hidden') == 'true';
  }
};

/*  Status: too slow on 30k files


// QN_SyncManager   ---   Track changes in various instances of QuickNote.
// Desc: There always is an instance of QN, that was last focused. It has the
// most recent data, and is identified with a special attribute on the <page>
// element. Other instances search for this "special", or "latest" instance
// and copy data from there.
var QN_SyncManager = {
  // marks given (XULDocument) QuickNote instance as "latest" if aLatest == true
  // Note: this function does not remove the "latest" attribute from other
  // instances automatically, *you* should take care of it.
  setLatest : function(aDoc, aLatest) {
    if(!aDoc) return;
    var qnPage = aDoc.getElementById('quicknote-main');
//    dump("Latest -> " + aLatest + "\n");
    if(aLatest)
      qnPage.setAttribute('latest', 'true');
    else
      qnPage.removeAttribute('latest');
  },

  // returns true, if given QN instance is "latest"
  isLatest : function(aDoc) {
    if(!aDoc) return false;
    var qnPage = aDoc.getElementById('quicknote-main');
    return (qnPage.getAttribute('latest') == 'true');
  },

  // search for the "latest" instance with QuickNote_EnumQNWindows()
  // returns the document of the found instance, null if nothing found
  _latest : null,
  findLatest : function() {
    this._latest = null;
    QN_Enumerator.EnumQNWindows(this._callback);
    return this._latest;
  },

  _callback : function(aDoc, aType) {
    if(QN_SyncManager.isLatest(aDoc)) {
      if(!QN_SyncManager._latest)
        QN_SyncManager._latest = aDoc;
      else // just in case
        QN_SyncManager.setLatest(aDoc, false);
    }
    return true;
  },

  // copy data from aFrom QN instance to aTo.
  copy : function(aFrom, aTo) {
    if(!aFrom || !aTo) return;
    dump('<QN_SyncManager.copy()>  write me!\n');

    var i;
    for(i=1; i<=4; i++) {
      var padFrom = aFrom.getElementById('Pad'+i);
      var padTo = aTo.getElementById('Pad'+i);
      if(!padFrom || !padTo) continue;
      if(padTo.value != padFrom.value) // todo: preserve view position
        padTo.value = padFrom.value;
    }
  }
};

// [called onfocus]
// check if any other QN window contains more recent data
function QN_synchronize(aCurrentDoc) {
  if(!aCurrentDoc || QN_SyncManager.isLatest(aCurrentDoc)) return;
  // find the "latest" instance and copy data from there
  var qnLatest = QN_SyncManager.findLatest();
  dump(1);
  QN_SyncManager.copy(qnLatest, aCurrentDoc);
  QN_SyncManager.setLatest(qnLatest, false);
  QN_SyncManager.setLatest(aCurrentDoc, true);
}
*/

function QuickNote_Test() {
  QN_Enumerator.EnumQNWindows(function(d,t) {alert(d.location+'\n'+t);return true;});
}

//All SpellBound code by Robert (deepestsender@exchangecode.com)
function setupSpellBound() {
  // determine if spellbound is available - if not do nothing.
  if (typeof(Components.classes["@mozilla.org/spellbound;1"]) != 'undefined') {
    // define the commandset, menu, and keyset to use with spellbound
    const commandset_id  = "maincommands";
    const keyset_id      = "mainkeys";
    const menu_id        = "clipmenu";
 
    // append spellbound command to existing commandset
    var parent = document.getElementById(commandset_id);
    var element = document.createElement("command");
    element.setAttribute("id", "cmd_QNspellbound");
 
    // you can pass a node such as a textbox, textarea, input, or editor
    element.setAttribute("oncommand", "spellBound_useNode();");
    // or you can pass a string of text
    //element.setAttribute("oncommand", "spellBound_useText();");
    parent.appendChild(element);
 
    // append spellbound stringbundle to the parent of commandset
    element = document.createElement("stringbundle");
    element.setAttribute("id", "sb_QNspellbound");
    element.setAttribute("src", "chrome://spellbound/locale/spellboundOverlay.properties");
    parent.parentNode.appendChild(element);
    var sbSpellBound = document.getElementById("sb_QNspellbound");
 
    // append spellbound key to existing keyset
    parent = document.getElementById(keyset_id);
    element = document.createElement("key");
    element.setAttribute("id", "key_QNspellbound");
    // the keycode may conflict with an accesskey for your extension and may change per locale
    element.setAttribute("keycode", sbSpellBound.getString("editcheckspelling.keybinding"));
    element.setAttribute("command", "cmd_QNspellbound");
    element.setAttribute("modifiers", "accel,shift");
    parent.appendChild(element);
 
    // append spellbound menuseparator to existing menu
    parent = document.getElementById(menu_id);
    element = document.createElement("menuseparator");
    parent.appendChild(element);
 
    // append spellbound menuitem to existing menu
    element = document.createElement("menuitem");
    element.setAttribute("id", "menu_spellbound");
    element.setAttribute("observes", "cmd_QNspellbound");
    element.setAttribute("label", sbSpellBound.getString("checkSpellingCmd.label"));
    element.setAttribute("key", "key_QNspellbound");
    // the accesskey may conflict with an accesskey for your extension and may change per locale
    element.setAttribute("accesskey", sbSpellBound.getString("editcheckspelling.accesskey"));
    element.setAttribute("command", "cmd_QNspellbound");
    parent.appendChild(element);
  }
}
function spellBound_useNode() {
  var args = [];
  var Note = QN_Document.getElementById("qntabs").selectedIndex+1;//current note number
  args[0] = QN_Document.getElementById('Pad'+Note)
  window.openDialog("chrome://spellbound/content/SBSpellCheck.xul",
"_blank","chrome,close,titlebar,modal", false, false, true, args);
}


function spellBound_useText() {
  var Note = QN_Document.getElementById("qntabs").selectedIndex+1;//current note number
  var data = QuickNote_getTextareaText(Note);
  var args = [];
  var results = [];
  args[0] = data;
  window.openDialog("chrome://spellbound/content/SBSpellCheck.xul", "_blank","chrome,close,titlebar,modal", false, false, true, args, results);
  QuickNote_setTextareaText(results[0], Note);
}


/*
////////////////////////////////////////////////////////////////////////////////
// mailIt
//  **********NOT IN USE YEST*******
// Parameters:
//  targetNodeID: where the data is to save!
// Returns: true
// To-Do: TESTING, can't get to get Thunderbird to load a Compose window
// Haven't tested since TB 0.1. Will do after 0.6 is released
////////////////////////////////////////////////////////////////////////////////
function QuickNote_qnMailWindow(targetNodeID) {
  var QNmailText = QuickNote_getTextareaText(targetNodeID).value;
  window.openDialog("chrome://quicknote/content/mail.xul",
      "_blank", "chrome,all,dialog", QNmailText);
  return true;
}
*/

